<?php
/**
 * Database Class provides an API to communicate with SQLite Database.
 * Nearly all methods are identical to my other Oracle and MySQL classes.
 * This is the PHP 5 only class, the old PHP 4 class won't be extended anymore!
 * Requires dbdefs.inc.php for global access data (dbname,appname).
 * @author Sascha 'SieGeL' Pfalz <php@saschapfalz.de>
 * @package db_SQLite
 * @version 0.20 (07-Aug-2010)
 * $Id: db_sqlite.class.php,v 1.2 2010/08/07 17:47:03 siegel Exp $
 * @see dbdefs.inc.php
 * @license http://opensource.org/licenses/bsd-license.php BSD License
 * @filesource
 */

/**
 * SQLite class.
 * @package db_SQLite
 */
class db_SQLite
  {
  /**
   * Class version.
   * @private
   * @var string
   */
  private $classversion = '0.20';

  /**
   * Internal connection handle.
   * @access protected
   * @var resource
   */
  protected $sock = NULL;

  /**
   * The Database filename.
   * @access protected
   * @var string
   */
  protected $database = '';

  /**
   * The default database filename permission mode.
   * @access protected
   * @var integer
   */
  protected $db_mode = 0666;

  /**
   * The Name of the application using this class.
   * @access protected
   * @var string
   */
  protected $appname = '';

  /**
   * Contains the actual query to be processed.
   * @access protected
   * @var string
   */
  protected $currentQuery = '';

  /**
   * Flag indicates how the class should interact with errors
   * @access protected
   * @var integer
   */
  protected $showError = db_SQLite::DBOF_SHOW_NO_ERRORS;

  /**
   * Debugstate, default is OFF.
   * @access protected
   * @var integer
   */
  protected $debug = db_SQLite::DBOF_DEBUGOFF;

  /**
   * The SAPI type of php (used to detect CLI sapi)
   * @private
   * @var string
   */
  private $SAPI_type = '';

  /**
   * Email Address for the administrator of this project
   * @private
   * @var string
   */
  private $AdminEmail = '';

  /**
   * Error code of last sqlite operation (set in Print_Error()).
   * @private
   * @var integer
   */
  private $myErrno = 0;

  /**
   * Error string of last sqlite operation (set in Print_Error()).
   * @private
   * @var string
   */
  var $myErrStr = '';

  /**
   * Set to TRUE if Connect() should use persistant connection, else new one (Default)
   * @private
   * @var boolean
   */
  private $usePConnect = FALSE;

  /**
   * How many queries where executed.
   * @private
   * @var integer
   */
  private $querycounter = 0;

  /**
   * Amount of time spent executing class methods.
   * @private
   * @var integer
   */
  private $querytime = 0.000;

  /** DEBUG: No Debug Info */
  const DBOF_DEBUGOFF           = 1;
  /** DEBUG: Debug to screen */
  const DBOF_DEBUGSCREEN        = 2;
  /** DEBUG: Debug to error.log */
  const DBOF_DEBUGFILE          = 4;

  /**#@+
   * Connect and error handling.
   * If NO_ERRORS is set and an error occures, the class still reports an
   * an error of course but the error shown is reduced to avoid showing
   * sensible informations in a productive environment.
   * Set RETURN_ALL_ERRORS if you want to handle errors yourself.
   */
  const DBOF_SHOW_NO_ERRORS     = 0;
  const DBOF_SHOW_ALL_ERRORS    = 1;
  const DBOF_RETURN_ALL_ERRORS  = 2;
  /**#@-*/

  /**#@+
   * All defines for custom error codes *
   */
  const SQLITEDB_ERR_NO_APPNAME_DEFINE  = -1;
  const SQLITEDB_ERR_NO_CONNECTION      = -2;
  const SQLITEDB_ERR_NO_ACTIVE_CONNECT  = -3;
  const SQLITEDB_ERR_SQL_SYNTAX_ERROR   = -4;
  const SQLITEDB_ERR_EXTENSION_MISSING  = -5;
  /**#@-*/

  /**
   * Constructor of class.
   * @param string $ext_config Pass here the full name to your define file where all external class defines are set. If empty uses "dbdefs.inc.php".
   */
  public function __construct($ext_config = '')
    {
    $this->SAPI_type  = @php_sapi_name();
    $this->AdminEmail = isset($_SERVER['SERVER_ADMIN']) ? strip_tags($_SERVER['SERVER_ADMIN']) : '';
    if($ext_config == '')
      {
      require_once('dbdefs.inc.php');
      }
    else
      {
      require_once($ext_config);
      }
    if(function_exists('sqlite_open') == FALSE)
      {
      $this->setErrorHandling(db_SQLite::DBOF_SHOW_ALL_ERRORS);
      $this->myErrStr = 'Missing function sqlite_open() - extension not loaded!';
      $this->myErrno  = db_SQLite::SQLITEDB_ERR_EXTENSION_MISSING;
      $this->Print_Error("SQLite Extension not loaded!");
      }
    if(defined('SQLITEDB_APPNAME')==false)
      {
      $this->myErrno  = db_SQLite::SQLITEDB_ERR_NO_APPNAME_DEFINE;
      $this->myErrStr = 'Define SQLITEDB_APPNAME not set!';
      $this->appname  = '!!! NOT SET !!!';
      $this->setErrorHandling(db_SQLite::DBOF_SHOW_ALL_ERRORS);
      $this->Print_Error('dbdefs.inc.php not found/wrong configured! Please check Class installation!');
      }
    $this->appname = SQLITEDB_APPNAME;
    if(defined('SQLITEDB_ERRORMODE'))                       // You can set a default behavour for error handling in debdefs.inc.php
      {
      $this->SetErrorHandling(SQLITEDB_ERRORMODE);
      }
    else
      {
      $this->SetErrorHandling(db_SQLite::DBOF_SHOW_NO_ERRORS);         // Default is not to show too much informations
      }
    } // __construct()

  /**
   * Performs the connection to a SQLite database file.
   * If anything goes wrong calls Print_Error().
   * You should set the defaults for your connection by setting
   * database filename in dbdefs.inc.php and leave connect() parameters empty.
   * @param string $database Optional Filename of database to use
   * @param string $mode Optional the file protection bits as octal number, defaults to 0666.
   * @return mixed Either the DB connection handle or NULL in case of an error.
   * @see dbdefs.inc.php
   * @see sqlite_open()
   * @see sqlite_popen()
   */
  public function Connect($database = '', $mode = 0)
    {
    if($this->sock)
      {
      return($this->sock);
      }
    if($database!='')
      {
      $this->database = $database;
      }
    else
      {
      $this->database = SQLITEDB_DEFAULT_DB;
      }
    if($mode != 0)
      {
      $this->db_mode = $mode;
      }
    $start = $this->GetMicrotime();
    if($this->usePConnect == FALSE)
      {
      $this->PrintDebug('sqlite_open("'.$this->database.'",'.$this->db_mode.')');
      $rc = @sqlite_open($this->database,$this->db_mode,$sqlite_errmsg);
      }
    else
      {
      $this->PrintDebug('sqlite_popen("'.$this->database.'",'.$this->db_mode.')');
      $rc = @sqlite_popen($this->database,$this->db_mode,$sqlite_errmsg);
      }
    if(is_resource($rc) == false)
      {
      $this->myErrStr = $sqlite_errmsg;
      $this->myErrno  = db_SQLite::SQLITEDB_ERR_NO_CONNECTION;
      $this->Print_Error('Connect(): Connection to '.$this->database.' failed!');
      return(0);
      }
    $this->sock = $rc;
    return($this->sock);
    } // Connect()

  /**
   * Disconnects from SQLite database.
   * You may optionally pass an external link identifier.
   * @param mixed $other_sock Optionally your own connection handle to close, else internal will be used
   * @see sqlite_close()
   */
  public function Disconnect($other_sock=-1)
    {
    if($other_sock!=-1)
      {
      @sqlite_close($other_sock);
      }
    else
      {
      if($this->sock)
        {
        @sqlite_close($this->sock);
        $this->sock = 0;
        }
      }
    $this->currentQuery = '';
    }

  /**
   * Single-Row query method.
   * Returns only the first row of a given query!
   * Resflag can be one of SQLITE_NUM or SQLITE_ASSOC or SQLITE_BOTH depending on what kind of array you want to be returned.
   * @param string $querystring The query to be executed.
   * @param integer $resflag Decides how the result should be returned:
   *  - SQLITE_ASSOC = Data is returned as associative array (default value).
   *  - SQLITE_NUM   = Data is returned as numbered array.
   *  - SQLITE_BOTH  = Data is returned both as numbered and as associative array.
   * @param integer $no_exit Decides how the class should react on errors.
   *                         If you set this to 1 the class won't automatically exit on an error but instead return the sqlite_errno value.
   *                         Default of 0 means that the class calls Print_Error() and exists.
   * @return mixed Either the result of the query or an error code or no return value at all
   * @see Print_Error()
   * @see sqlite_unbuffered_query()
   * @see sqlite_fetch_array()
   */
  public function Query($querystring,$resflag = SQLITE_ASSOC, $no_exit = 0)
    {
    if(!$this->sock)
      {
      $this->myErrStr = '';
      $this->myErrno  = db_SQLite::SQLITEDB_ERR_NO_ACTIVE_CONNECT;
      return($this->Print_Error('Query(): No active connection!',$querystring));
      }
    if($this->debug)
      {
      $this->PrintDebug($querystring);
      }
    $this->currentQuery = $querystring;
    if($this->showError == db_SQLite::DBOF_RETURN_ALL_ERRORS)
      {
      $no_exit = 1;  // Override if user has set master define
      }
    $start = $this->GetMicrotime();
    $this->querycounter++;
    $res = @sqlite_unbuffered_query($this->sock,$querystring,$resflag,$sqlite_errormsg);
    if($res == false)
      {
      if($no_exit)
        {
        $this->myErrno  = db_SQLite::SQLITEDB_ERR_SQL_SYNTAX_ERROR;
        $this->myErrStr = $sqlite_errormsg;
        return($this->myErrno);
        }
      else
        {
        return($this->Print_Error($sqlite_errormsg));
        }
      }
    if(@sqlite_has_more($res) == true)
      {
      $retarr = @sqlite_fetch_array($res,$resflag);
      $this->querytime+= ($this->GetMicrotime() - $start);
      return($retarr);
      }
    else
      {
      $this->querytime+= ($this->GetMicrotime() - $start);
      return($res);
      }
    }

  /**
   * Performs a multi-row query and returns result identifier.
   * @param string $querystring The Query to be executed
   * @param integer $no_exit The error indicator flag, can be one of:
   *  - 0 = (Default), In case of an error Print_Error is called and script terminates
   *  - 1 = In case of an error this function returns the error from sqlite_last_error()
   * @return mixed A resource identifier or an errorcode (if $no_exit = 1)
   * @see sqlite_query()
   */
  public function QueryResult($querystring, $no_exit = 0)
    {
    if(!$this->sock)
      {
      $this->myErrStr = '';
      $this->myErrno  = db_SQLite::SQLITEDB_ERR_NO_ACTIVE_CONNECT;
      return($this->Print_Error('QueryResult(): No active Connection!',$querystring));
      }
    if($this->debug)
      {
      $this->PrintDebug($querystring);
      }
    $this->currentQuery = $querystring;
    $start = $this->GetMicrotime();
    $this->querycounter++;
    $res = @sqlite_query($this->sock,$querystring);
    if($res == false)
      {
      if($no_exit)
        {
        $this->myErrno  = db_SQLite::SQLITEDB_ERR_SQL_SYNTAX_ERROR;
        $this->myErrStr = $sqlite_errormsg;
        return($this->myErrno);
        }
      else
        {
        return($this->Print_Error($sqlite_errormsg));
        }
      }
    $this->querytime+= ($this->GetMicrotime() - $start);
    return($res);
    }

  /**
   * Fetches next row from result handle.
   * Returns either numeric array (SQL_NUM), associative array (SQL_ASSOC) or both (SQLITE_BOTH) for one data row as pointed to by result var.
   * @param mixed $result The resource identifier as returned by QueryResult()
   * @param integer $resflag How you want the data to be returned:
   *  - SQLITE_ASSOC = Data is returned as associative array.
   *  - SQLITE_NUM   = Data is returned as numbered array.
   *  - SQLITE_BOTH  = Data is returned both as associative and numbered array.
   * @return array One row of the resulting query or NULL if there is no data anymore to read.
   * @see sqlite_fetch_array()
   * @see QueryResult()
   */
  public function FetchResult($result,$resflag = SQLITE_ASSOC)
    {
    if(!$result)
      {
      return($this->Print_Error('FetchResult(): No valid result handle!'));
      }
    $start = $this->GetMicrotime();
    $resar = @sqlite_fetch_array($result,$resflag);
    $this->querytime+= ($this->GetMicrotime() - $start);
    return($resar);
    }

  /**
   * Frees result returned by QueryResult().
   * Note that SQLite has nothing to free (?) so here only the internal query variable is set to an empty string.
   * @param mixed $result The resource identifier you want to be freed.
   * @see QueryResult()
   * @see FetchResult()
   */
  public function FreeResult($result)
    {
    $this->currentQuery = '';
    }

  /**
   * Returns number of rows affected by most recent DML operation against $dbhandle.
   * @param resource $sock Optional a connection handle, if none given the internal socket will be used.
   * @return integer Number of affected rows.
   * @see NumRows()
   * @see sqlite_changes()
   */
  public function AffectedRows($sock = -1)
    {
    $rows = 0;

    if($sock = -1)
      {
      $rows = @sqlite_changes($this->sock);
      }
    else
      {
      $rows = @sqlite_changes($sock);
      }
    return($rows);
    }

  /**
   * Returns number of rows in a result set.
   * Note: This only works for statements returning a result set, for DML operations use AffectedRows() !
   * @param resource $stmt The resource as returned from QueryResult()
   * @see AffectedRows()
   * @see sqlite_num_rows()
   */
  public function NumRows($result)
    {
    return(sqlite_num_rows($result));
    }

  /**
   * Returns last used auto_increment id.
   * @param mixed $extsock Optionally an external SQLite socket to use. If not given the internal socket is used.
   * @return integer The last automatic insert id that was assigned by the SQLite server.
   * @see sqlite_last_insert_rowid()
   */
  public function LastInsertId($extsock=-1)
    {
    if($extsock==-1)
      {
      return(@sqlite_last_insert_rowid($this->sock));
      }
    else
      {
      return(@sqlite_last_insert_rowid($extsock));
      }
    }

  /**
   * Commits current transaction.
   * Note: Requires BEGIN TRANSACTION first!
   * Without BEGIN TRANSACTION an auto-transaction is always auto-commited!
   */
  function Commit()
    {
    if($this->debug)
      {
      $this->PrintDebug('COMMIT called');
      }
    $this->Query('COMMIT TRANSACTION');
    }

  /**
   * Rollback current transaction.
   * Note: Requires BEGIN TRANSACTION first!
   * Without BEGIN TRANSACTION an auto-transaction is always auto-commited!
   */
  function Rollback()
    {
    if($this->debug)
      {
      $this->PrintDebug('ROLLBACK called');
      }
    $this->Query('ROLLBACK TRANSACTION');
    }

  /**
   * Allows to set the handling of errors.
   *
   * - db_SQLite::DBOF_SHOW_NO_ERRORS    => Show no security-relevant informations
   * - db_SQLite::DBOF_SHOW_ALL_ERRORS   => Show all errors (useful for develop)
   * - db_SQLite::DBOF_RETURN_ALL_ERRORS => No error/autoexit, just return the mysql_error code.
   * @param integer $val The Error Handling mode you wish to use.
   */
  public function SetErrorHandling($val)
    {
    $this->showError = $val;
    }

  /**
   * Returns the current error handling mode.
   * @return integer The current error handling mode.
   * @see SetErrorHandling()
   * @since 0.20
   */
  public function GetErrorHandling()
    {
    return($this->showError);
    }

  /**
   * Function allows debugging of SQL Queries.
   * $state can have these values:
   * - db_SQLite::DBOF_DEBUGOFF    = Turn off debugging
   * - db_SQLite::DBOF_DEBUGSCREEN = Turn on debugging on screen (every Query will be dumped on screen)
   * - db_SQLite::DBOF_DEBUFILE    = Turn on debugging on PHP errorlog
   * You can mix the debug levels by adding the according defines!
   * @param integer $state The DEBUG Level you want to be set
   * @see GetDebug()
   */
  public function SetDebug($state)
    {
    $this->debug = $state;
    }

  /**
   * Returns the current debug setting.
   * @return integer The debug setting (bitmask)
   * @see SetDebug()
   */
  public function GetDebug()
    {
    return($this->debug);
    }

  /**
   * Handles output according to internal debug flag.
   * @param string $msg The Text to be included in the debug message.
   * @see error_log()
   * @see SetDebug()
   */
  public function PrintDebug($msg)
    {
    if(!$this->debug)
      {
      return;
      }
    if($this->SAPI_type != 'cli')
      {
      $formatstr = "<div align=\"left\" style=\"background-color:#ffffff; color:#000000;z-index:10000;\"><pre>DEBUG: %s</pre></div>\n";
      }
    else
      {
      $formatstr =  "DEBUG: %s\n";
      }
    if($this->debug & db_SQLite::DBOF_DEBUGSCREEN)
      {
      @printf($formatstr,$msg);
      }
    if($this->debug & db_SQLite::DBOF_DEBUGFILE)
      {
      @error_log('DEBUG: '.$msg,0);
      }
    }

  /**
   * Retrieve last SQLite error number.
   * @param mixed $other_sock Optionally your own connection handle to check, else internal will be used
   * @return integer The SQLite error number of the last operation
   * @see sqlite_last_error
   */
  public function GetErrno($other_sock = -1)
    {
    if( $other_sock == -1 )
      {
      if(!$this->sock)
        {
        return($this->myErrno);
        }
      else
        {
        return(@sqlite_last_error($this->sock));
        }
      }
    else
      {
      if(!$other_sock)
        {
        return($this->myErrno);
        }
      else
        {
        return(@sqlite_last_error($other_sock));
        }
      }
    }

  /**
   * Retrieve last SQLite error description.
   * @param mixed $other_sock Optionally your own connection handle to check, else internal will be used
   * @return string The SQLite error description of the last operation
   * @see mysql_error
   */
  public function GetErrorText($other_sock = -1)
    {
    if( $other_sock == -1 )
      {
      if(!$this->sock)
        {
        return($this->myErrStr);
        }
      else
        {
        return(@sqlite_error_string($this->GetErrno()));
        }
      }
    else
      {
      if(!$other_sock)
        {
        return($this->myErrStr);
        }
      else
        {
        return(@sqlite_error_string($this->GetErrno($other_sock)));
        }
      }
    }

  /**
   * Prints out SQLite Error in own <div> container and exits.
   * Please note that this function does not return as long as you have not set DBOF_RETURN_ALL_ERRORS!
   * @param string $ustr User-defined Error string to show
   * @param mixed $var2dump Optionally a variable to print out with print_r()
   * @see print_r()
   * @see sqlite_last_error()
   * @see sqlite_error_string()
   */
  public function Print_Error($ustr="",$var2dump="")
    {
    if($this->sock)
      {
      $errnum = @sqlite_last_error($this->sock);
      $errstr = @sqlite_error_string($errnum);
      }
    else
      {
      $errnum = $this->myErrno;
      $errstr = $this->myErrStr;
      }
    $this->myErrno  = $errnum;
    $this->myErrStr = $errstr;
    $filename = basename($_SERVER['SCRIPT_FILENAME']);
    if($errstr=='')
      {
      $errstr = 'N/A';
      }
    if($errnum=='')
      {
      $errnum = -1;
      }
    @error_log($this->appname.': SQLite class error in '.$filename.': '.$ustr.' ('.chop($errstr).')',0);
    if($this->showError == db_SQLite::DBOF_RETURN_ALL_ERRORS)
      {
      return($errnum);      // Return the error number
      }
    $this->SendMailOnError($errnum,$errstr,$ustr);
    $crlf = "\n";
    $space= " ";
    if($this->SAPI_type != 'cli')
      {
      $crlf = "<br>\n";
      $space= "&nbsp;";
      echo("<br>\n<div align=\"left\" style=\"background-color: #EEEEEE; color:#000000;\">\n");
      echo("<font color=\"red\" face=\"Arial, Sans-Serif\"><b>".$this->appname.": SQLite class error occured!</b></font><br>\n<br>\n<code>\n");
      }
    else
      {
      echo("\n!!! ".$this->appname.": SQLite class error occured !!!\n\n");
      }
    echo($space."CODE: ".$errnum.$crlf);
    echo($space."DESC: ".$errstr.$crlf);
    echo($space."FILE: ".$filename.$crlf);
    if($this->showError == db_SQLite::DBOF_SHOW_ALL_ERRORS)
      {
      if($this->currentQuery!="")
        {
        echo("QUERY: ".$this->currentQuery.$crlf);
        }
      echo($space."QCNT: ".$this->querycounter.$crlf);
      if($ustr!='')
        {
        echo($space."INFO: ".$ustr.$crlf);
        }
      if($var2dump!='')
        {
        echo($space.'DUMP: ');
        if(is_array($var2dump))
          {
          if($this->SAPI_type != 'cli')
            {
            echo('<pre>');
            print_r($var2dump);
            echo("</pre>\n");
            }
          else
            {
            print_r($var2dump);
            }
          }
        else
          {
          echo($var2dump.$crlf);
          }
        }
      }
    if($this->SAPI_type != 'cli')
      {
      if($this->AdminEmail != '')
        {
        echo("<br>\nPlease inform <a href=\"mailto:".$this->AdminEmail."\">".$this->AdminEmail."</a> about this problem.");
        echo("</code>\n");
        echo("</div>\n");
        }
      echo("<div align=\"right\"><small>PHP v".phpversion()." / SQLite Class v".$this->classversion."</small></div>\n");
      }
    else
      {
      if($this->AdminEmail != '')
        {
        echo("\nPlease inform ".$this->AdminEmail." about this problem.\n");
        }
      echo("\nRunning on PHP v".phpversion()." / SQLite Class v".$this->classversion."\n");
      }
    $this->Disconnect();
    exit;
    }

  /**
   * Send error email if programmer has defined a valid email address and enabled it with the define SQLITEDB_SENTMAILONERROR.
   * @param integer $merrno SQLite errno number
   * @param string $merrstr SQLite error description
   * @param string $uerrstr User-supplied error description
   * @see dbdefs.inc.php
   * @see mail()
   */
  private function SendMailOnError($merrno,$merrstr,$uerrstr)
    {
    if(!defined('SQLITEDB_SENTMAILONERROR') || SQLITEDB_SENTMAILONERROR == 0 || $this->AdminEmail == '')
      {
      return;
      }
    $sname    = (isset($_SERVER['SERVER_NAME']) == TRUE) ? $_SERVER['SERVER_NAME'] : '';
    $saddr    = (isset($_SERVER['SERVER_ADDR']) == TRUE) ? $_SERVER['SERVER_ADDR'] : '';
    $raddr    = (isset($_SERVER['REMOTE_ADDR']) == TRUE) ? $_SERVER['REMOTE_ADDR'] : '';
    $uagent   = (isset($_SERVER['HTTP_USER_AGENT'])) ? $_SERVER['HTTP_USER_AGENT'] : '';
    if($sname == '')
      {
      $server = 'n/a';
      }
    else
      {
      $server  = $sname.' ('.$saddr.')';
      }
    if($uagent == '')
      {
      $uagent = 'n/a';
      }
    if($raddr == '')
      {
      $clientip = 'n/a';
      }
    else
      {
      $clientip = $raddr." (".@gethostbyaddr($raddr).")";
      }
    $message = "SQLite Class v".$this->classversion.": Error occured on ".date('r')." !!!\n\n";
    $message.= "      APPLICATION: ".$this->appname."\n";
    $message.= "  AFFECTED SERVER: ".$server."\n";
    $message.= "       USER AGENT: ".$uagent."\n";
    $message.= "       PHP SCRIPT: ".$_SERVER['SCRIPT_FILENAME']."\n";
    $message.= "   REMOTE IP ADDR: ".$clientip."\n";
    $message.= "    DATABASE FILE: ".$this->database."\n";
    $message.= "SQL ERROR MESSAGE: ".$merrstr."\n";
    $message.= "   SQL ERROR CODE: ".$merrno."\n";
    $message.= "    QUERY COUNTER: ".$this->querycounter."\n";
    $message.= "         INFOTEXT: ".$uerrstr."\n";
    if($this->currentQuery != '')
      {
      $message.= "        SQL QUERY:\n";
      $message.= "------------------------------------------------------------------------------------\n";
      $message.= $this->currentQuery."\n";
      }
    $message.= "------------------------------------------------------------------------------------\n";
    if(defined('MYSQLDB_MAIL_EXTRAARGS') && MYSQLDB_MAIL_EXTRAARGS != '')
      {
      @mail($this->AdminEmail,'SQLite Class v'.$this->classversion.' ERROR #'.$merrno.' OCCURED!',$message,MYSQLDB_MAIL_EXTRAARGS);
      }
    else
      {
      @mail($this->AdminEmail,'SQLite Class v'.$this->classversion.' ERROR #'.$merrno.' OCCURED!',$message);
      }
    }

  /**
   * Returns version of this class.
   * @return string The version of this class.
   */
  public function GetClassVersion()
    {
    return($this->classversion);
    }

  /**
   * Returns SQLite library version.
   * @return string SQLite library version.
   */
  public function Version()
    {
    return(@sqlite_libversion());
    }

  /**
   * Escapes a given string with the 'sqlite_escape_string' method.
   * Always use this function to avoid SQL injections when adding dynamic data to SQLite!
   * This function also handles the settings for magic_quotes_gpc, if
   * this setting is enabled it will call stripslashes() first.
   * @param string $str The string to escape.
   * @return string The escaped string.
   */
  public function EscapeString($str)
    {
    $data = $str;
    if(@get_magic_quotes_gpc())
      {
      $data = @stripslashes($data);
      }
    return(@sqlite_escape_string($data));
    }

  /**
   * Returns amount of queries executed by this class.
   * @return integer Querycount
   */
  public function GetQueryCount()
    {
    return($this->querycounter);
    }

  /**
   * Returns amount of time spend on queries executed by this class.
   * @return float Time in seconds.msecs spent in executin MySQL code.
   */
  public function GetQueryTime()
    {
    return($this->querytime);
    }

   /**
   * Returns microtime in format s.mmmmm.
   * Used to measure SQL execution time.
   * @static
   * @return float the current time in microseconds.
   */
  public static function GetMicrotime()
    {
    list($usec, $sec) = explode(" ",microtime());
    return (floatval($usec) + floatval($sec));
    }

  } // End-of-class
?>
